@@ -249,8 +249,7 @@ class Servlet(metaclass=ABCMeta):
             n_batches += 1
             batch_size_max = max(batch_size_max, n)
             batch_size_min = min(batch_size_min, n)
-            batch_size_mean = (batch_size_mean *
-                               (n_batches - 1) + n) / n_batches
+            batch_size_mean = (batch_size_mean * (n_batches - 1) + n) / n_batches
             if n_batches % 1000 == 0:
                 logger.info('batch size stats (count, max, min, mean): %d, %d, %d, %.1f',
                             n_batches, batch_size_max, batch_size_min, batch_size_mean)
@@ -301,7 +300,7 @@ class Servlet(metaclass=ABCMeta):
 
 
 class MPServer(metaclass=ABCMeta):
-    MP_CLASS = mp
+    MP_CLASS = mp.get_context('spawn')
     # This class attribute is provided because in some cases
     # one may want to use `torch.multiprocessing`, which is
     # a drop-in replacement for the standard `multiprocessing`
@@ -359,6 +358,10 @@ class MPServer(metaclass=ABCMeta):
             n = len(self._servlets)
             k = 0
             for m in self._servlets:
+                print('servlet')
+                print(m)
+                print('')
+
                 m.start()
                 z = self._q_err.get()
                 assert z == 'ready'
@@ -381,9 +384,11 @@ class MPServer(metaclass=ABCMeta):
                 k += 1
                 logger.info(f"servlet processes ready: {k}/{n}")
 
-        self._t_gather_output = threading.Thread(target=self._gather_output)
+        self._t_gather_output = threading.Thread(
+            target=self._gather_output, name='ResultCollector')
         self._t_gather_output.start()
-        self._t_gather_error = threading.Thread(target=self._gather_error)
+        self._t_gather_error = threading.Thread(
+            target=self._gather_error, name='ErrorCollector')
         self._t_gather_error.start()
 
     def stop(self):
@@ -406,9 +411,7 @@ class MPServer(metaclass=ABCMeta):
         # Reset CPU affinity.
         psutil.Process().cpu_affinity(cpus=[])
 
-    async def async_call(self,
-                         x,
-                         *,
+    async def async_call(self, x, /, *,
                          enqueue_timeout: Union[int, float] = None,
                          total_timeout: Union[int, float] = None,
                          ):
@@ -422,9 +425,7 @@ class MPServer(metaclass=ABCMeta):
             uid=uid, fut=fut,
             t0=time0, total_timeout=total_timeout)
 
-    def call(self,
-             x,
-             *,
+    def call(self, x, /, *,
              enqueue_timeout: Union[int, float] = None,
              total_timeout: Union[int, float] = None,
              ):
@@ -446,7 +447,7 @@ class MPServer(metaclass=ABCMeta):
         return self._call_wait_for_result(
             uid=uid, fut=fut, t0=time0, total_timeout=total_timeout)
 
-    def async_stream(self, data_stream, *,
+    def async_stream(self, data_stream, /, *,
                      return_exceptions: bool = False,
                      return_x: bool = False,
                      ) -> streamer.AsyncStream:
@@ -519,7 +520,7 @@ class MPServer(metaclass=ABCMeta):
             )
         )
 
-    def stream(self, data_stream, *,
+    def stream(self, data_stream, /, *,
                return_exceptions: bool = False,
                return_x: bool = False,
                ) -> streamer.Stream:
@@ -612,21 +613,17 @@ class MPServer(metaclass=ABCMeta):
 
         cpus = self._resolve_cpus(cpus=cpus, workers=workers)
 
-        name = name or 'servlet'
+        name = name or 'Servlet'
 
         for cpu in cpus:
-            if cpu is None:
-                # Not pinned to any core.
-                logger.info('adding servlet %s', servlet.__name__)
-            else:
-                # Pinned to the specified cpu core.
-                logger.info('adding servlet %s at CPU %s',
-                            servlet.__name__, cpu)
+            # Pinned to the specified cpu core.
+            logger.info('adding servlet %s at CPU %s',
+                        servlet.__name__, cpu)
 
             self._servlets.append(
                 self.MP_CLASS.Process(
                     target=servlet.run,
-                    name=f'{name}-{cpu}',
+                    name=f"{name}-{','.join(map(str, cpu))}",
                     kwargs={
                         'q_in': q_in,
                         'q_out': q_out,
@@ -641,7 +638,7 @@ class MPServer(metaclass=ABCMeta):
     def _resolve_cpus(self, *, cpus: list = None, workers: int = None):
         n_cpus = psutil.cpu_count(logical=True)
 
-        # Either `workers` or `cpus`, but not both.
+        # Either `workers` or `cpus`, but not both,
         # can be specified.
         if workers:
             # Number of workers is specified.
@@ -653,6 +650,7 @@ class MPServer(metaclass=ABCMeta):
             assert 0 < workers <= n_cpus * 4
             cpus = list(reversed(range(n_cpus))) * 4
             cpus = sorted(cpus[:workers])
+            # List[int]
         elif cpus:
             assert isinstance(cpus, list)
             # Create as many processes as the length of `cpus`.
@@ -666,6 +664,7 @@ class MPServer(metaclass=ABCMeta):
             # Create as many processes as there are cores,
             # one process pinned to one core.
             cpus = list(range(n_cpus))
+            # List[int]
 
         for i, cpu in enumerate(cpus):
             if isinstance(cpu, int):
@@ -966,6 +965,16 @@ class EnsembleServer(MPServer):
         return self._q_in
 
 
+class SimpleServlet(Servlet):
+    def __init__(self, *, func, batch_size: int = None, **kwargs):
+        super().__init__(batch_size=batch_size)
+        self._kwargs = kwargs
+        self._func = func
+
+    def call(self, x):
+        return self._func(x, **self._kwargs)
+
+
 class SimpleServer(SequentialServer):
     '''
     One worker process per CPU core, each worker running
@@ -973,8 +982,7 @@ class SimpleServer(SequentialServer):
     '''
 
     def __init__(self,
-                 func: Callable,
-                 *,
+                 func: Callable, /, *,
                  max_queue_size: int = None,
                  cpus: list = None,
                  workers: int = None,
@@ -983,21 +991,16 @@ class SimpleServer(SequentialServer):
                  ):
         '''
         `func`: a function that takes an input value,
-        which will be the value provided in calls to the server,
-        plus `kwargs`.
+            which will be the value provided in calls to the server,
+            plus `kwargs`. This function must be defined on the module
+            level (can't be defined within a function), and can't be
+            a lambda.
         '''
         super().__init__(max_queue_size=max_queue_size)
 
-        class SimpleServlet(Servlet):
-            def __init__(self, *, batch_size: int = None, **kwargs):
-                super().__init__(batch_size=batch_size)
-                self._kwargs = kwargs
-
-            def call(self, x):
-                return func(x, **self._kwargs)
-
         self.add_servlet(SimpleServlet,
                          cpus=cpus,
                          workers=workers,
                          batch_size=batch_size,
+                         func=func,
                          **kwargs)
